from unittest import mock

import pytest

from infrahouse_core.aws.route53.exceptions import IHRecordNotFound, IHZoneNotFound
from infrahouse_core.aws.route53.zone import Zone


def test_zone_name():
    zone = Zone(zone_name="ci-cd.infrahouse.com")
    assert zone.zone_name == "ci-cd.infrahouse.com."


def test_zone_id():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert zone.zone_id == "Z06790082OU0POPWOCPZ4"

    mock_client = mock.Mock()
    mock_client.list_hosted_zones_by_name.return_value = {
        "HostedZones": [
            {
                "Id": "/hostedzone/foo",
                "Name": "ci-cd.infrahouse.com.",
                "CallerReference": "terraform-20230810002609851800000001",
                "Config": {"Comment": "Managed by Terraform", "PrivateZone": False},
                "ResourceRecordSetCount": 5,
            }
        ],
        "DNSName": "ci-cd.infrahouse.com",
        "IsTruncated": False,
        "MaxItems": "1",
    }
    with mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client):
        zone = Zone(zone_name="ci-cd.infrahouse.com")
        assert zone.zone_id == "foo"
        mock_client.list_hosted_zones_by_name.assert_called_once_with(DNSName="ci-cd.infrahouse.com.", MaxItems="1")


def test_zone_id_not_found():
    mock_client = mock.Mock()
    mock_client.list_hosted_zones_by_name.return_value = {
        "HostedZones": [
            {
                "Id": "/hostedzone/foo",
                "Name": "ci-cd.infrahouse.com.",
                "CallerReference": "terraform-20230810002609851800000001",
                "Config": {"Comment": "Managed by Terraform", "PrivateZone": False},
                "ResourceRecordSetCount": 5,
            }
        ],
        "DNSName": "ci-cd.infrahouse.com",
        "IsTruncated": False,
        "MaxItems": "1",
    }
    with mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client):
        zone = Zone(zone_name="foo_zone")
        with pytest.raises(IHZoneNotFound):
            assert zone.zone_id == "foo"
        mock_client.list_hosted_zones_by_name.assert_called_once_with(DNSName="foo_zone.", MaxItems="1")


def test_zone_name_by_id():
    mock_client = mock.Mock()
    mock_client.get_hosted_zone.return_value = {
        "HostedZone": {
            "Id": "/hostedzone/foo",
            "Name": "ci-cd.infrahouse.com.",
            "CallerReference": "terraform-20230810002609851800000001",
            "Config": {"Comment": "Managed by Terraform", "PrivateZone": False},
            "ResourceRecordSetCount": 5,
        },
        "DelegationSet": {
            "NameServers": [
                "ns-261.awsdns-32.com",
                "ns-1795.awsdns-32.co.uk",
                "ns-776.awsdns-33.net",
                "ns-1311.awsdns-35.org",
            ]
        },
    }
    with mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client):
        zone = Zone(zone_id="foo")
        assert zone.zone_name == "ci-cd.infrahouse.com."
        mock_client.get_hosted_zone.assert_called_once_with(Id="foo")


@pytest.mark.parametrize(
    "response, result",
    [
        (
            {
                "ResourceRecordSets": [
                    {
                        "Name": "ip-10-1-100-243.ci-cd.infrahouse.com.",
                        "Type": "A",
                        "TTL": 300,
                        "ResourceRecords": [{"Value": "10.1.100.243"}],
                    }
                ],
                "IsTruncated": True,
                "NextRecordName": "ip-10-1-101-82.ci-cd.infrahouse.com.",
                "NextRecordType": "A",
                "MaxItems": "1",
            },
            ["10.1.100.243"],
        ),
        (
            {
                "ResourceRecordSets": [
                    {
                        "Name": "ip-10-1-100-243.ci-cd.infrahouse.com.",
                        "Type": "A",
                        "TTL": 300,
                        "ResourceRecords": [{"Value": "10.1.100.243"}, {"Value": "10.1.1.1"}],
                    }
                ],
                "IsTruncated": True,
                "NextRecordName": "ip-10-1-101-82.ci-cd.infrahouse.com.",
                "NextRecordType": "A",
                "MaxItems": "1",
            },
            ["10.1.100.243", "10.1.1.1"],
        ),
    ],
)
def test_search_hostname(response, result):
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert zone.search_hostname("ip-10-1-100-243") == [
    #     "10.1.100.243"
    # ]
    # return
    mock_client = mock.Mock()
    mock_client.list_resource_record_sets.return_value = response
    with (
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
    ):
        zone = Zone(zone_name="ci-cd.infrahouse.com")
        assert zone.search_hostname("ip-10-1-100-243") == result
        mock_client.list_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id",
            StartRecordName="ip-10-1-100-243.ci-cd.infrahouse.com.",
            StartRecordType="A",
            MaxItems="1",
        )


def test_search_hostname_not_found():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert zone.search_hostname("foo") == ["10.1.100.243"]
    mock_client = mock.Mock()
    mock_client.list_resource_record_sets.return_value = {
        "ResourceRecordSets": [
            {
                "Name": "ip-10-1-100-243.ci-cd.infrahouse.com.",
                "Type": "A",
                "TTL": 300,
                "ResourceRecords": [{"Value": "10.1.100.243"}],
            }
        ],
        "NextRecordName": "ip-10-1-101-82.ci-cd.infrahouse.com.",
        "NextRecordType": "A",
        "MaxItems": "1",
    }
    with (
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
    ):
        zone = Zone(zone_name="ci-cd.infrahouse.com")
        with pytest.raises(IHRecordNotFound):
            assert zone.search_hostname("foo")
        mock_client.list_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id", StartRecordName="foo.ci-cd.infrahouse.com.", StartRecordType="A", MaxItems="1"
        )


def test_add_one_record():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # zone.add_record("foo", "10.1.1.1")
    # assert zone.search_hostname("foo") == ["10.1.1.1"]
    # zone.add_record("foo", "10.1.1.2")
    # assert zone.search_hostname("foo") == ["10.1.1.1", "10.1.1.2"]
    mock_client = mock.Mock()
    with (
        mock.patch.object(Zone, "search_hostname", side_effect=IHRecordNotFound),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
    ):
        zone = Zone(zone_name="foo.com")
        zone.add_record("foo", "10.1.1.1")
        mock_client.change_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id",
            ChangeBatch={
                "Changes": [
                    {
                        "Action": "CREATE",
                        "ResourceRecordSet": {
                            "Name": "foo.foo.com.",
                            "Type": "A",
                            "TTL": 300,
                            "ResourceRecords": [{"Value": "10.1.1.1"}],
                        },
                    }
                ]
            },
        )


def test_add_second_record():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # zone.add_record("foo", "10.1.1.1")
    # assert zone.search_hostname("foo") == ["10.1.1.1"]
    # zone.add_record("foo", "10.1.1.2")
    # assert zone.search_hostname("foo") == ["10.1.1.1", "10.1.1.2"]
    mock_client = mock.Mock()
    with (
        mock.patch.object(Zone, "search_hostname", return_value=["10.1.1.1"]),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
    ):
        zone = Zone(zone_name="foo.com")
        zone.add_record("foo", "10.1.1.2")
        mock_client.change_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id",
            ChangeBatch={
                "Changes": [
                    {
                        "Action": "UPSERT",
                        "ResourceRecordSet": {
                            "Name": "foo.foo.com.",
                            "Type": "A",
                            "TTL": 300,
                            "ResourceRecords": [{"Value": "10.1.1.1"}, {"Value": "10.1.1.2"}],
                        },
                    }
                ]
            },
        )


def test_delete_first_record():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert sorted(zone.search_hostname("foo")) == sorted(["10.1.1.1", "10.1.1.2"])
    # zone.delete_record("foo", "10.1.1.1")
    # assert zone.search_hostname("foo") == ["10.1.1.2"]
    # zone.delete_record("foo", "10.1.1.2")
    # with pytest.raises(IHRecordNotFound):
    #     assert zone.search_hostname("foo")
    mock_client = mock.Mock()
    with (
        mock.patch.object(Zone, "search_hostname", return_value=["10.1.1.1", "10.1.1.2"]),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
        mock.patch.object(Zone, "_get_record_ttl", return_value=300),
    ):
        zone = Zone(zone_name="foo.com")
        zone.delete_record("foo", "10.1.1.1")
        mock_client.change_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id",
            ChangeBatch={
                "Changes": [
                    {
                        "Action": "UPSERT",
                        "ResourceRecordSet": {
                            "Name": "foo.foo.com.",
                            "Type": "A",
                            "TTL": 300,
                            "ResourceRecords": [{"Value": "10.1.1.2"}],
                        },
                    }
                ]
            },
        )


def test_delete_last_record():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert sorted(zone.search_hostname("foo")) == sorted(["10.1.1.1", "10.1.1.2"])
    # zone.delete_record("foo", "10.1.1.1")
    # assert zone.search_hostname("foo") == ["10.1.1.2"]
    # zone.delete_record("foo", "10.1.1.2")
    # with pytest.raises(IHRecordNotFound):
    #     assert zone.search_hostname("foo")
    mock_client = mock.Mock()
    with (
        mock.patch.object(Zone, "search_hostname", return_value=["10.1.1.1"]),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
        mock.patch.object(Zone, "_get_record_ttl", return_value=300),
    ):
        zone = Zone(zone_name="foo.com")
        zone.delete_record("foo", "10.1.1.1")
        mock_client.change_resource_record_sets.assert_called_once_with(
            HostedZoneId="foo_id",
            ChangeBatch={
                "Changes": [
                    {
                        "Action": "DELETE",
                        "ResourceRecordSet": {
                            "Name": "foo.foo.com.",
                            "Type": "A",
                            "TTL": 300,
                            "ResourceRecords": [{"Value": "10.1.1.1"}],
                        },
                    }
                ]
            },
        )


def test_delete_non_existing_record():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert sorted(zone.search_hostname("foo")) == sorted(["10.1.1.1", "10.1.1.2"])
    # zone.delete_record("foo", "10.1.1.1")
    # assert zone.search_hostname("foo") == ["10.1.1.2"]
    # zone.delete_record("foo", "10.1.1.2")
    # with pytest.raises(IHRecordNotFound):
    #     assert zone.search_hostname("foo")
    mock_client = mock.Mock()
    with (
        mock.patch.object(Zone, "search_hostname", return_value=["10.1.1.1"]),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
        mock.patch.object(Zone, "_get_record_ttl", return_value=300),
    ):
        zone = Zone(zone_name="foo.com")
        zone.delete_record("foo", "10.1.1.2")
        mock_client.change_resource_record_sets.assert_not_called()


def test_search_hostname_new():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert zone.search_hostname("ip-10-1-102-189") == ["10.1.100.243"]
    mock_response = {"ResourceRecordSets": [], "IsTruncated": False, "MaxItems": "1"}
    mock_client = mock.Mock()
    mock_client.list_resource_record_sets.return_value = mock_response
    with (
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "zone_id", new_callable=mock.PropertyMock, return_value="foo_id"),
    ):
        zone = Zone(zone_name="foo.com")
        with pytest.raises(IHRecordNotFound):
            zone.search_hostname("ip-10-1-102-189")


def test_delete_record_unknown():
    # zone = Zone(zone_name="ci-cd.infrahouse.com")
    # assert zone.search_hostname("ip-10-1-102-189") == ["10.1.100.243"]
    mock_client = mock.Mock()

    with (
        mock.patch.object(Zone, "search_hostname", side_effect=IHRecordNotFound) as mock_search_hostname,
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client),
        mock.patch.object(Zone, "_client", new_callable=mock.PropertyMock, return_value=mock_client) as mc,
    ):
        zone = Zone(zone_name="foo.com", zone_id="foo_id")
        zone.delete_record("ip-10-1-102-189", "10.1.102.189")
        mock_search_hostname.assert_called_once_with("ip-10-1-102-189")
        mc.change_resource_record_sets.assert_not_called()



from unittest import mock

from infrahouse_core.aws.config import AWSConfig


def test_aws_home():
    with mock.patch("infrahouse_core.aws.config.osp.expanduser", return_value="/home/foo"):
        aws = AWSConfig()
        assert aws.aws_home == "/home/foo/.aws"



"""
Module for ASG class - a class to work with Autoscaling group.
"""

from logging import getLogger
from typing import Dict, List

from botocore.exceptions import ClientError

from infrahouse_core.aws import get_client
from infrahouse_core.aws.asg_instance import ASGInstance

LOG = getLogger()


class ASG:
    """AWS Autoscaling group."""

    def __init__(self, asg_name: str, region: str = None):
        self._asg_name = asg_name
        self._region = region

    @property
    def instance_refreshes(self) -> List[Dict]:
        """
        :return: List of ASG instance refresh tasks.
        """
        return self._autoscaling_client.describe_instance_refreshes(
            AutoScalingGroupName=self._asg_name,
        )["InstanceRefreshes"]

    @property
    def instances(self) -> List[ASGInstance]:
        """
        :return: List of EC2 instances in the autoscaling group.
        """
        return [
            ASGInstance(instance_id=instance["InstanceId"], region=self._region)
            for instance in self._describe_auto_scaling_groups["AutoScalingGroups"][0]["Instances"]
        ]

    def cancel_instance_refresh(self):
        """Cancel all instance refreshes."""
        try:
            self._autoscaling_client.cancel_instance_refresh(AutoScalingGroupName=self._asg_name)
        except ClientError as err:
            if err.response["Error"]["Code"] == "ActiveInstanceRefreshNotFound":
                LOG.warning(err)
            else:
                raise

    def complete_lifecycle_action(self, hook_name="terminating", result="CONTINUE", instance_id=None):
        """
        Completes the lifecycle hook.
        See details on https://docs.aws.amazon.com/autoscaling/ec2/userguide/completing-lifecycle-hooks.html

        :param hook_name: Hook name.
        :type hook_name: str
        :param result: Result of the hook. Can be either CONTINUE or ABANDON.
        :type result: str
        :param instance_id: EC2 instance_id for which complete the hook.
            If not given, assume the local instance.
        :type instance_id: str
        """
        self._autoscaling_client.complete_lifecycle_action(
            LifecycleHookName=hook_name,
            AutoScalingGroupName=self._asg_name,
            LifecycleActionResult=result,
            InstanceId=instance_id or ASGInstance().instance_id,
        )

    def record_lifecycle_action_heartbeat(self, hook_name="terminating", instance_id=None):
        """
        Extend lifecycle wait time.

        :param hook_name: Hook name.
        :type hook_name: str
        :param instance_id: EC2 instance_id for which the hook is waiting.
            If not given, assume the local instance.
        :type instance_id: str
        """
        self._autoscaling_client.record_lifecycle_action_heartbeat(
            LifecycleHookName=hook_name,
            AutoScalingGroupName=self._asg_name,
            InstanceId=instance_id or ASGInstance().instance_id,
        )

    @property
    def _autoscaling_client(self):
        return get_client("autoscaling", region=self._region)

    @property
    def _describe_auto_scaling_groups(self):
        return self._autoscaling_client.describe_auto_scaling_groups(
            AutoScalingGroupNames=[
                self._asg_name,
            ],
        )



from textwrap import dedent

import pytest

from infrahouse_core.aws.config import AWSConfig

CONTENT = dedent(
    """
    [default]
    region = us-west-123

    [profile foo]
    region = us-west-345

    [profile bar]
    """
)

CONTENT_2 = dedent(
    """
    [profile foo]
    region = us-west-345

    [profile bar]
    """
)


@pytest.mark.parametrize(
    "profile, content, region",
    [
        ("default", CONTENT, "us-west-123"),
        ("foo", CONTENT, "us-west-345"),
        ("bar", CONTENT, "us-west-123"),
        (None, CONTENT, "us-west-123"),
        ("default", CONTENT_2, None),
        ("foo", CONTENT_2, "us-west-345"),
        ("bar", CONTENT_2, None),
        (None, CONTENT_2, None),
    ],
)
def test_region(profile, region, content, tmpdir):
    aws_home = tmpdir.mkdir("home")
    cfg = aws_home.join("config")
    cfg.write(content)
    assert AWSConfig(aws_home=str(aws_home)).get_region(profile) == region
